@@ -11,15 +11,14 @@
 #' @param mu Integer, order of derivative.
 #' @param nu Integer, order of derivative.
 #' @param kernel_type String, the kernel.
+#' @param regularize Boolean.
 #' @return bandwidth sequence
 #' @keywords internal
-bw_rot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
+bw_rot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type, regularize){
   sd_y = stats::sd(y_data)
   sd_x = apply(x_data, 2, stats::sd)
   mx = apply(x_data, 2, mean)
   my = mean(y_data)
-  y_data = (y_data - my)/sd_y
-  x_data = sweep(x_data, 2, mx)/sd_x
   d = ncol(x_data)
   n = length(y_data)
   ng = length(y_grid)
@@ -94,6 +93,13 @@ bw_rot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
     h = (v_dgp/bias_dgp[, 3])^(1/alpha)*n^(-1/alpha)
     h = sd_y*sd_x*h
 
+    if (regularize == TRUE){
+      for (j in 1:ng){
+        h[j] <- max(h[j], sort(abs(x_data-x))[min(n, 20+q+4)])
+        h[j] <- max(h[j], sort(abs(y_data-y_grid[j]))[min(n, 20+p+4)])
+      }
+    }
+
   } else {
 
     bx = (4/(d+1))^(1/(d+4))*n^(-1/(d+4))*sd_x
@@ -171,7 +177,6 @@ bw_rot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
       v_dgp[, 1] = v_dgp * (Sx%*%Tx%*%Sx)[nu+1, nu+1]
     }
 
-    print('here')
     h = (v_dgp/bias_dgp[, 3])^(1/6)*n^(-1/6)
     h = stats::sd(y_data)*stats::sd(x_data)*h
 
@@ -192,15 +197,14 @@ bw_rot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
 #' @param mu Integer, order of derivative.
 #' @param nu Integer, order of derivative.
 #' @param kernel_type String, the kernel.
+#' @param regularize Boolean.
 #' @return bandwidth sequence
 #' @keywords internal
-bw_irot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
+bw_irot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type, regularize){
   sd_y = stats::sd(y_data)
   sd_x = apply(x_data, 2, stats::sd)
   mx = apply(x_data, 2, mean)
   my = mean(y_data)
-  y_data = (y_data - my)/sd_y
-  x_data = sweep(x_data, 2, mx)/sd_x
   d = ncol(x_data)
   n = length(y_data)
   ng = length(y_grid)
@@ -274,6 +278,13 @@ bw_irot = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
     h = (mean(v_dgp)/(2*mean(bias_dgp[, 3])))^(1/alpha)*n^(-1/alpha)
     h = sd_y*sd_x*h
 
+    if (regularize == TRUE){
+      h <- max(h, sort(abs(x_data-x))[min(n, 20+q+1)])
+      for(j in 1:ng){
+        h <- max(h, sort(abs(y_data-y_grid[j]))[min(n, 20+p+1)])
+      }
+    }
+
   } else {
     # assuming product kernel
 
@@ -487,8 +498,8 @@ bw_mse = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
       }
       #TODO: Fix cx estimation
       #estimating theta hats
-      print(normal_dgps(y_grid[j], mu, my, sd_y))
-      print(normal_dgps(x, 2, mx, sd_x))
+      #print(normal_dgps(y_grid[j], mu, my, sd_y))
+      #print(normal_dgps(x, 2, mx, sd_x))
       bias_dgp[j, 1] = normal_dgps(y_grid[j], mu, my, sd_y) * prod(normal_dgps(x, 2, mx, sd_x))
       bias_dgp[j, 2] = normal_dgps(y_grid[j], p+1, my, sd_y) * prod(normal_dgps(x, 0, mx, sd_x))
       mv = mvec(q+1, d)
@@ -514,7 +525,7 @@ bw_mse = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
 
       bias_dgp[j, 3] = (bias_dgp[j, 1] + bias_dgp[j, 2])^2
     }
-    print(bias_dgp)
+    #print(bias_dgp)
 
     # variance estimate. See Lemma 7 in the Appendix.
     v_dgp = matrix(0, ncol=1, nrow=ng)
@@ -584,9 +595,6 @@ bw_imse = function(y_data, x_data, y_grid, x, p, q, mu, nu, kernel_type){
   sd_x = apply(x_data, 2, stats::sd)
   mx = apply(x_data, 2, mean)
   my = mean(y_data)
-  y_data = (y_data - my)/sd_y
-  x_data = sweep(x_data, 2, mx)/sd_x
-
   d = ncol(x_data)
   n = length(y_data)
   ng = length(y_grid)
